import logging
import os
import pathlib
import shutil

import markdown
import frontmatter
import pyquery

from .config import (
    templates,
    default_template,
    content_selector,
    public,
    pages,
    static,
    extra_data,
)


def insert_content(template: os.PathLike, mapping: dict[str, str]):
    document = pyquery.PyQuery(filename=template)
    if not document("head > title"):
        document("head").append("<title/>")
    for selector, content in {**extra_data, **mapping}.items():
        element = document.find(selector)
        if not element and selector != "template":
            logging.warning(
                f"No matches found for selector `{selector}` in template `{template}`"
            )
        element.append(content)
    return document.outer_html()


def process_file(file_path: os.PathLike):
    with open(file_path) as f:
        post = frontmatter.load(f)
    post[content_selector] = markdown.markdown(post.content)
    template = templates / post.get("template", default_template)
    return insert_content(template, post.metadata)


def process_folder(directory: os.PathLike):
    for dirpath, _, files in os.walk(directory, followlinks=True):
        for file in files:
            if file.endswith(".md"):
                pubdir = public / pathlib.Path(dirpath).relative_to(directory)
                filename = file.replace(".md", ".html")
                pubdir.mkdir(parents=True, exist_ok=True)
                logging.info(f"Writing {pubdir / file}")
                with open(pubdir / filename, "w") as f:
                    f.write(process_file(os.path.join(dirpath, file)))


def build(clean=False):
    if clean:
        logging.info(f"Cleaning out existing public folder `{public}`")
        shutil.rmtree(public, ignore_errors=True)
    process_folder(pages)
    logging.info(f"Copying static folder `{static}`")
    shutil.copytree(static, public / static, dirs_exist_ok=True)



import typer
import pathlib

app = typer.Typer()


@app.callback()
def callback():
    """Welcome to Granita, a simple static site generator!

    If you're feeling kind you can buy me a coffee here 🍏 https://www.buymeacoffee.com/applephi"""


@app.command()
def init(name: str):
    """Initialize a new Granita project."""
    root = pathlib.Path(name)
    (root / "templates").mkdir(parents=True)
    (root / "static").mkdir(parents=True)
    (root / "pages").mkdir(parents=True)
    (root / "templates" / "default.html").write_text(
        """<html>
	<head>
		<link
			rel="stylesheet"
			href="https://unpkg.com/@picocss/pico@latest/css/pico.min.css"
		/>
	</head>
	<body>
		<div
			style="
				width: 60%;
				height: 50%;
				position: absolute;
				top: 0;
				bottom: 0;
				left: 0;
				right: 0;
				margin: auto;
			"
		>
			<content />
		</div>
	</body>
</html>
"""
    )
    (root / "pages" / "index.md").write_text(
        """---
title: "Granita"
---
# Hello world
Welcome to your first Granita project!

> *If you're feeling kind you can buy me a coffee [here](https://www.buymeacoffee.com/applephi)* 🍏"""
    )


@app.command()
def build(
    clean: bool = typer.Argument(
        False, help="Whether to clean out the public folder before building."
    )
):
    """Render templates to HTML."""
    from .transform import build

    build(clean)


if __name__ == "__main__":
    app(prog_name="granita")



import pathlib
import json
import logging

default = pathlib.Path(__file__).parent / "granita.config.json"
custom = pathlib.Path("granita.config.json")
with open(default) as f:
    data: dict = json.load(f)
if custom.exists():
    logging.info(f"Using custom configuration file {custom.absolute()}")
    with open(custom) as f:
        data.update(json.load(f))
else:
    logging.info(f"Using default configuration file {default.absolute()}")
logging.info("Using configuration:\n" + "\n".join(f"{k}: {v}" for k, v in data.items()))

pages = pathlib.Path(data["pages"])
static = pathlib.Path(data["static"])
public = pathlib.Path(data["public"])
templates = pathlib.Path(data["templates"])
default_template = data["default-template"]
content_selector = data["content-selector"]
extra_data = data["extra-data"]
